%{!?upstream_version: %global upstream_version %{version}%{?milestone}}
%global project kuryr
%global service kuryr-kubernetes
%global module kuryr_kubernetes

%global common_desc \
Kuryr Kubernetes provides a Controller that watches the Kubernetes API for \
Object changes and manages Neutron resources to provide the Kubernetes Cluster \
with OpenStack networking.

# %commit is intended to be set by tito builders at packaging time.
# The value in this spec file will not be kept up to date.
%global commit 8926a294348d3791040748774b0ac0892b968494

Name:           openshift-kuryr
Version:        4.14.0
Release:        202404151639.p0.g8926a29.assembly.stream%{?dist}
Summary:   OpenStack networking integration with OpenShift and Kubernetes
License:   ASL 2.0
URL:       http://docs.openstack.org/developer/kuryr-kubernetes/

Source0:        openshift-kuryr-4.14.0-202404151639.p0.g8926a29.assembly.stream.tar.gz
Source1:   kuryr.logrotate
Source2:   kuryr-controller.service
Source3:   openshift-kuryr.tmpfs
Source4:   kuryr-cni.service

BuildArch: noarch

Requires(pre): shadow-utils
%{?systemd_requires}

%description
[Maintainer] project: OCPBUGS, component: Networking / kuryr
Kuryr-Kubernetes brings OpenStack networking to OpenShift and Kubernetes clusters

%package -n python3-%{service}
Summary:        Kuryr Kubernetes libraries
%{?python_provide:%python_provide python3-%{service}}

BuildRequires:  git
BuildRequires:  python3-devel
BuildRequires:  python3-setuptools
BuildRequires:  python3-pbr
BuildRequires:  systemd-units

Requires:       python3-%{project}-lib >= 0.5.0
# NOTE(dulek): We should have pyroute2 0.5.7 here, but it's not avalable in
# repos we have access to. Sticking with 0.5.6 should be fine as long as we
# don't support SR-IOV stuff.
Requires:       python3-pyroute2 >= 0.5.7
Requires:       python3-requests >= 2.18.0
Requires:       python3-eventlet >= 0.22.0
Requires:       python3-oslo-cache >= 1.26.0
Requires:       python3-oslo-config >= 6.1.0
Requires:       python3-oslo-log >= 3.36.0
Requires:       python3-oslo-reports >= 1.18.0
Requires:       python3-oslo-serialization >= 2.18.0
Requires:       python3-oslo-service >= 1.40.2
Requires:       python3-oslo-utils >= 3.40.2
Requires:       python3-os-vif >= 1.12.0
Requires:       python3-stevedore >= 1.20.0
Requires:       python3-cotyledon >= 1.5.0
Requires:       python3-flask >= 0.12.3
Requires:       python3-retrying >= 1.2.3
Requires:       python3-openstacksdk >= 0.36.0
Requires:       python3-prettytable >= 0.7.2
Requires:       python3-prometheus_client >= 0.6.0
Requires:       python3-netaddr >= 0.7.19

%description -n python3-%{service}
[Maintainer] project: OCPBUGS, component: Networking / kuryr
%{common_desc}

This package contains the Kuryr Kubernetes Python library.

%package common
Summary:        Kuryr Kubernetes common files
Group:          Applications/System
Requires:   python3-%{service} = %{version}-%{release}

%description common
[Maintainer] project: OCPBUGS, component: Networking / kuryr
This package contains Kuryr files common to all services.

%package controller
Summary: Kuryr Kubernetes Controller
Requires: openshift-%{project}-common = %{version}-%{release}

%description controller
[Maintainer] project: OCPBUGS, component: Networking / kuryr
%{common_desc}

This package contains the Kuryr Kubernetes Controller that watches the
Kubernetes API and adds metadata to its Objects about the OpenStack resources
it obtains.

%package cni
Summary: CNI plugin
Requires: openshift-%{project}-common = %{version}-%{release}
%{?systemd_requires}

%description cni
[Maintainer] project: OCPBUGS, component: Networking / kuryr
%{common_desc}

This package contains the Kuryr Kubernetes Container Network Interface driver
that Kubelet calls to.

%prep
%autosetup -S git -n openshift-kuryr-4.14.0 -p1

find %{module} -name \*.py -exec sed -i '/\/usr\/bin\/env python/{d;q}' {} +

# Let's handle dependencies ourseleves
rm -f requirements.txt
rm -f test-requirements.txt
rm -f doc/requirements.txt

# Need to commit it to make sure no .devxyz will get added to version by pbr.
git commit -a --amend -m "Manage requirements on our own"
# Tagging will make sure pbr uses this as version.
git tag -a -m "%{version}" "%{version}"

# Kill egg-info in order to generate new SOURCES.txt
rm -rf kuryr_kubernetes.egg-info

%build
export REAL_GO_PATH=$(which go || true)
if [[ -n "$REAL_GO_PATH" ]]; then
    GOSHIM_DIR=/tmp/goshim
    mkdir -p $GOSHIM_DIR
    ln -s $REAL_GO_PATH $GOSHIM_DIR/go.real
    export PATH=$GOSHIM_DIR:$PATH
    # Use single quotes 'EOF' to avoid variable expansion.
cat > $GOSHIM_DIR/go << 'EOF'
#!/bin/sh
export GOEXPERIMENT=strictfipsruntime
export GOTOOLCHAIN=local

LOG_PREFIX="Go compliance shim [${__doozer_group}][${__doozer_key}]:"
echoerr() {
  echo -n "${LOG_PREFIX} " 1>&2
  cat <<< "$@" 1>&2
}

set -o xtrace

echo 1>&2
echo 1>&2
echo -n "${LOG_PREFIX} incoming command line arguments: " 1>&2
for arg in "$@"; do
  echo -n "\"${arg}\" " 1>&2
done
echo 1>&2
echo 1>&2

echoerr "----"
echoerr "incoming environment: "
echoerr "---------------------"
env 1>&2
echoerr "---------------------"
echo 1>&2

if [[ "${CGO_ENABLED}" == "0" ]]; then
    echoerr "non-compliant RPM attempted to use CGO_ENABLED=${CGO_ENABLED}"
fi

if [[ "${GOOS}" == "darwin" ]]; then
    echoerr "permitting CGO_ENABLED=${CGO_ENABLED} because of GOOS=${GOOS}"
else
    export CGO_ENABLED="1"
    echoerr "forcing CGO_ENABLED=${CGO_ENABLED}"
fi

HAS_TAGS=0
if cat <<< "$@" | grep "\-tags" > /dev/null; then
  HAS_TAGS="1"
fi

FORCE_FOD_MODE=1
FORCE_OPENSSL=1
FORCE_DYNAMIC=1
IN_RUN=0
IN_TAGS=0
ARGS=()  # We need to rebuild the argument list.
# Compilation with -extldflags "-static" is problematic with
# CGO_ENABLED=1 because compilation tries to link against
# static libraries which don't exist. Remove -static flag
# when detected. This is tricky because extldflags can be simple
# or something like -ldflags '-X $(REPO_PATH)/pkg/version.Raw=$(VERSION) -extldflags "-lm -lstdc++ -static"'
ARGS=()  # We need to rebuild the argument list.
for arg in "$@"; do
    if [[ "${arg}" == "run" ]]; then
      IN_RUN="1"
    fi

    # prior to detecting 'IN_RUN', grafana failed because it ran
    # 'go run build.go build' which caused the script to exec 'go run build.go build -tags strictfipsruntime'
    if [[ "${arg}" == "build" && "${IN_RUN}" == "0" ]]; then
      # -tags apparently cannot come after a build path
      # e.g. "build ./cmd/cluster-openshift-apiserver-operator -tags strictfipsruntime" is invalid.
      # So, if we see "build" and no "-tags" ahead, then go ahead and force FOD tag.

      ARGS+=("${arg}") # Add "build"
      if [[ "${FORCE_FOD_MODE}" == "1" && "${HAS_TAGS}" == "0" ]]; then
        ARGS+=("-tags")
        ARGS+=("strictfipsruntime")
      fi
      continue  # We've already added 'build', so don't reach the bottom of the loop where it would be added again.
    fi

    if [[ ( "${arg}" == "-tags="* || "${arg}" == "--tags="* ) && "${FORCE_FOD_MODE}" == "1" ]]; then
      echoerr "adding strictfipsruntime tag to \"${arg}\""
      arg=$(echo "${arg}" | tr -d "'" | tr -d "\"")  # Delete any quotes which get passed in literally. grafana managed this.
      if [[ "${arg}" == *" "* ]]; then  # If the tags parameter is space delimited
        arg="${arg} strictfipsruntime"
      else
        # Assume comma delimited
        arg="${arg},strictfipsruntime"
      fi
    fi

    if [[ "${IN_TAGS}" == "1" ]]; then
      if [[ "${FORCE_FOD_MODE}" == "1" ]]; then
        echoerr "adding strictfipsruntime tag to ${arg} (IN_TAGS=${IN_TAGS})"
        arg=$(echo "${arg}" | tr -d "'" | tr -d "\"")  # Delete any quotes which get passed in literally. prom-label-proxy managed this.
        if [[ "${arg}" == *" "* ]]; then  # If the tags parameter is space delimited
          arg="${arg} strictfipsruntime"
        else
          # Assume comma delimited
          arg="${arg},strictfipsruntime"
        fi
      fi
      if [[ "${FORCE_OPENSSL}" == "1" ]]; then
        pre_arg="${arg}"
        arg=$(echo "${arg}" | sed 's/no_openssl/shim_prevented_no_openssl/g')
        if [[ "${pre_arg}" != "${arg}" ]]; then
          echoerr "non-compliant: eliminated no_openssl"
        fi
      fi
      IN_TAGS=0
    fi

    if [[ "${arg}" == "-tags" || "${arg}" == "--tags" ]]; then
      IN_TAGS=1
    fi

    # Compilation with -extldflags "-static" is problematic with
    # CGO_ENABLED=1 because compilation tries to link against
    # static libraries which don't exist. Remove -static flag
    # when detected. This is tricky because extldflags can be simple
    # or something like -ldflags '-X $(REPO_PATH)/pkg/version.Raw=$(VERSION) -extldflags "-lm -lstdc++ -static"'
    # Note that extldflags is a flag embedded within the value of the
    # -ldflags argument. From our script's perspective, it will be part of a single
    # argument, but this argument might look like '-X $(REPO_PATH)/pkg/version.Raw=$(VERSION) -extldflags "-lm -lstdc++ -static"'.
    if [[ "${arg}" == *"-extldflags"* && "${FORCE_DYNAMIC}" == "1" ]]; then
      # We replace -static with -lc because '-lc' implies to link against stdlib. This is a default
      # and should therefore be benign for virtually any compilation (unless -nostdlib or -nodefaultlibs
      # is specified -- and we don't account for this).
      # Why replace instead of remove? Consider the complex possible scenarios:
      # -ldflags '-extldflags "-static"'   # Removing -extldflags would mean we also need to remove ldflags.
      # -ldflags '-X $(REPO_PATH)/pkg/version.Raw=$(VERSION) -extldflags "-static"'  # Would remove extldflags but keep ldflags
      # -ldflags '-X $(REPO_PATH)/pkg/version.Raw=$(VERSION) -extldflags "-static -lm"'  # Would need to remove -static but keep extldflags
      # In any scenario, replacing "-static" with something benign should work without the need for complex logic.
      pre_arg="${arg}"
      arg=$(echo "${arg}" | sed "s/-static/-lc/g")
      if [[ "${pre_arg}" != "${arg}" ]]; then
        echoerr "non-compliant: eliminated static"
      fi
    fi
    ARGS+=("${arg}")
done

echo 1>&2
echo 1>&2
echo -n "${LOG_PREFIX} final command line arguments: " 1>&2
for arg in "${ARGS[@]}"; do
  echo -n "\"${arg}\" " 1>&2
done
echo 1>&2
echo 1>&2

echoerr "Invoking actual go binary"
SCRIPT_DIR=$( cd -- "$( dirname -- "${BASH_SOURCE[0]}" )" &> /dev/null && pwd ) # Get the directory in which this script is executing
${SCRIPT_DIR}/go.real "${ARGS[@]}"

EOF
    chmod +x $GOSHIM_DIR/go
fi

%py3_build

%install
%py3_install

# Create config files directories
install -d -m 755 %{buildroot}%{_sysconfdir}/%{project}
install -d -m 755 %{buildroot}%{_localstatedir}/log/%{project}

# Install logrotate
install -p -D -m 644 %{SOURCE1} %{buildroot}%{_sysconfdir}/logrotate.d/openshift-%{service}

# Install systemd units
install -p -D -m 644 %{SOURCE2} %{buildroot}%{_unitdir}/kuryr-controller.service
install -p -D -m 644 %{SOURCE4} %{buildroot}%{_unitdir}/kuryr-cni.service

# Kuryr run directories
install -p -D -m 644 %{SOURCE3} %{buildroot}%{_tmpfilesdir}/openshift-kuryr.conf
install -d %{buildroot}%{_localstatedir}/run/kuryr

# Kuryr cni_ds_init
install -d -m 755 %{buildroot}%{_libexecdir}/%{project}
install -p -D -m 755 cni_ds_init %{buildroot}%{_libexecdir}/%{project}/
install -p -D -m 755 etc/cni/net.d/10-kuryr.conflist %{buildroot}%{_sysconfdir}/%{project}-cni/10-kuryr.conflist

%pre -n python3-%{service}
getent group %{project} >/dev/null || groupadd -r %{project}
getent passwd %{project} >/dev/null || \
    useradd -r -g %{project} -d %{_sharedstatedir}/%{project} -s /sbin/nologin \
    -c "OpenStack Kuryr Daemons" %{project}
exit 0

%post controller
%systemd_post kuryr-controller.service

%preun controller
%systemd_preun kuryr-controller.service

%postun controller
%systemd_postun_with_restart kuryr-controller.service

%post cni
%systemd_post kuryr-cni.service

%preun cni
%systemd_preun kuryr-cni.service

%postun cni
%systemd_postun_with_restart kuryr-cni.service

%files controller
%license LICENSE
%{_bindir}/kuryr-k8s-controller
%{_bindir}/kuryr-k8s-sanity
%{_bindir}/kuryr-k8s-status
%{_bindir}/kuryr-gather-openstack-data
%{_unitdir}/kuryr-controller.service

%files -n python3-%{service}
%license LICENSE
%{python3_sitelib}/%{module}
%{python3_sitelib}/%{module}-*.egg-info
%exclude %{python3_sitelib}/%{module}/tests

%files common
%license LICENSE
%doc README.rst
%dir %attr(0755, %{project}, %{project}) %{_sysconfdir}/%{project}
%config(noreplace) %{_sysconfdir}/logrotate.d/*
%dir %attr(0755, %{project}, %{project}) %{_localstatedir}/log/%{project}
%{_tmpfilesdir}/openshift-kuryr.conf
%dir %attr(0755, %{project}, %{project}) %{_localstatedir}/run/kuryr

%files cni
%license LICENSE
%{_bindir}/kuryr-cni
%{_bindir}/kuryr-daemon
%{_unitdir}/kuryr-cni.service
%dir %attr(0755, root, root) %{_libexecdir}/%{project}
%{_libexecdir}/%{project}/cni_ds_init
%config(noreplace) %attr(0640, root, %{project}) %{_sysconfdir}/%{project}-cni/10-kuryr.conflist
